/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/bluetooth/bluetooth.h>
#include <bluetooth/mesh/models.h>
#include <bluetooth/mesh/dk_prov.h>
#include <dk_buttons_and_leds.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/zephyr.h>
#include "DS18B20.h"
#include "OneWire.h"
#include "stdio.h"
#include "model_handler.h"

#define DS18B20
#define LED1_NODE DT_NODELABEL(led1)
#define LED_GPIO DT_GPIO_LABEL(LED1_NODE, gpios)
#define LED_PIN DT_GPIO_PIN(LED1_NODE, gpios)
#define LED_FLAGS DT_GPIO_FLAGS(LED1_NODE, gpios)

#define CHANNEL_COUNT (4)   // ADS131B04-Q1 -> 4 Channels

///////////////////////////////////
// Variables for sensor evaluation
///////////////////////////////////
volatile uint16_t Timer_100usec;
volatile uint16_t ReadyCnt;
volatile bool RequestADread;
volatile bool Flag1msec;
volatile bool ReadyInterrupt;
static bool BluetoothInitialized = false;
volatile uint8_t RXbyte;

float Temperature[4];

typedef struct
{
  int32_t uV;
  int32_t max;
  int32_t min;
  int32_t diff;
  int32_t Mem[MEANSIZE];
  int32_t Sum;
  int32_t maxvar;
  int32_t minvar;
} Mean_t;

Mean_t Mean[4];

typedef struct
{
  int32_t uV;
  int32_t max;
  int32_t min;
  int32_t diff;
  float   mV;
  int32_t Mem[AVGSIZE];
  int32_t Sum;
  int32_t maxvar;
  int32_t minvar;
} Avg_t;

Avg_t Avg[4];

typedef struct
{
    uint16_t response;
    uint16_t crc;
    uint16_t crc_read;
    int32_t channel[CHANNEL_COUNT];
} adc_channel_data;

adc_channel_data ADresult;

// 1.1V Result 7668064
const int32_t Bit24Max = 8388607;
const int32_t SumMax = Bit24Max * AVGSIZE;

//
//  FUNCTIONS
//
void sensor_handle()
{
  static uint32_t counter = 0;
  static uint16_t avgindex = 0;
  static uint16_t meanindex = 0;

  if (ReadyInterrupt) // RequestADread
  {
    ReadyInterrupt = false;
    avgindex++;
    if (avgindex >= AVGSIZE)
    {
      avgindex = 0;
      for (uint16_t ch = 0; ch < 4; ch++)
      {
        Avg[ch].max = Avg[ch].maxvar;
        Avg[ch].min = Avg[ch].minvar;
        Avg[ch].diff = Avg[ch].max - Avg[ch].min;
        Avg[ch].maxvar = -2000000;
        Avg[ch].minvar = 2000000;

        Mean[ch].Sum += Avg[ch].uV;
        Mean[ch].Sum -= Mean[ch].Mem[meanindex];
        Mean[ch].Mem[meanindex] = Avg[ch].uV;

        Mean[ch].uV = Mean[ch].Sum / MEANSIZE;
        if (Mean[ch].maxvar < Mean[ch].uV)
          Mean[ch].maxvar = Mean[ch].uV;
        if (Mean[ch].minvar > Mean[ch].uV)
          Mean[ch].minvar = Mean[ch].uV;
      }
      meanindex++;
      if (meanindex >= MEANSIZE)
      {
#ifdef DS18B20
        DS18B20_ReadAll(); // 2sec
        DS18B20_StartAll();
#endif
        meanindex = 0;
        for (uint16_t ch = 0; ch < 4; ch++)
        {
          Mean[ch].max = Mean[ch].maxvar;
          Mean[ch].min = Mean[ch].minvar;
          Mean[ch].diff = Mean[ch].max - Mean[ch].min;
          Mean[ch].maxvar = -2000000;
          Mean[ch].minvar = 2000000;
#ifdef DS18B20
          DS18B20_GetTemperature(ch, &Temperature[ch]);
#endif
        }
#ifdef DS18B20
        char TempInfo[50] = {};
        sprintf(TempInfo, "%5.2f %5.2f %5.2f %5.2f\r\n", Temperature[0], Temperature[1], Temperature[2], Temperature[3]);
        printk("%5.2f %5.2f %5.2f %5.2f\r\n", Temperature[0], Temperature[1], Temperature[2], Temperature[3]);

#endif
      }
    }
  }
}

static void bt_ready(int err)
{
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}

	printk("Bluetooth initialized\n");

	//dk_leds_init();
	//dk_buttons_init(NULL);

	err = bt_mesh_init(bt_mesh_dk_prov_init(), model_handler_init());
	if (err) {
		printk("Initializing mesh failed (err %d)\n", err);
		return;
	}

	if (IS_ENABLED(CONFIG_SETTINGS)) {
		err = settings_load();
    if(err)
    {
      printk("Initializing mesh failed (err %d)\n", err);
      return;
    }
  }

	/* This will be a no-op if settings_load() loaded provisioning info */
	err = bt_mesh_prov_enable(BT_MESH_PROV_ADV | BT_MESH_PROV_GATT);
  if(err)
  {
    printk("Error Code: %i\n");
  }

	printk("Mesh initialized\n");
  BluetoothInitialized = true;
}

void main(void)
{	
	//DS18B20_Init(DS18B20_Resolution_12bits);

	int err;
	err = bt_enable(bt_ready);
	if (err) 
  {
		printk("Bluetooth init failed (err %d)\n", err);
	}

  /*
	for(;;)
	{
		sensor_handle();
		k_sleep(Z_TIMEOUT_US(200));
		ReadyInterrupt = true;
  }
  */
  
}
